summaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
-rw-r--r--module/zfs/metaslab.c16
1 files changed, 14 insertions, 2 deletions
diff --git a/module/zfs/metaslab.c b/module/zfs/metaslab.c
index a14057f89..5da929b48 100644
--- a/module/zfs/metaslab.c
+++ b/module/zfs/metaslab.c
@@ -2485,6 +2485,18 @@ metaslab_activate(metaslab_t *msp, int allocator, uint64_t activation_weight)
return (0);
}
+ /*
+ * If the metaslab has literally 0 space, it will have weight 0. In
+ * that case, don't bother activating it. This can happen if the
+ * metaslab had space during find_valid_metaslab, but another thread
+ * loaded it and used all that space while we were waiting to grab the
+ * lock.
+ */
+ if (msp->ms_weight == 0) {
+ ASSERT0(range_tree_space(msp->ms_allocatable));
+ return (SET_ERROR(ENOSPC));
+ }
+
if ((error = metaslab_activate_allocator(msp->ms_group, msp,
allocator, activation_weight)) != 0) {
return (error);
@@ -3735,8 +3747,8 @@ metaslab_group_alloc_normal(metaslab_group_t *mg, zio_alloc_list_t *zal,
* worse metaslab (we waited for that metaslab to be loaded
* after all).
*
* If the activation failed due to an I/O error we skip to
* the next metaslab.
*/
boolean_t activated;
if (activation_error == 0) {
@@ -3735,8 +3747,8 @@ metaslab_group_alloc_normal(metaslab_group_t *mg, zio_alloc_list_t *zal,
* after all).
*
- * If the activation failed due to an I/O error we skip to
- * the next metaslab.
+ * If the activation failed due to an I/O error or ENOSPC we
+ * skip to the next metaslab.
*/
boolean_t activated;